@@ -49,10 +49,9 @@ class Field(object):
 
         :param path: JSON field to fetch the value from. Either a string,
             or a list of strings in case of a nested field.
-        :param required: whether this field is required. Missing required
-            fields result in MissingAttributeError.
+        :param required: whether this field is required. Missing required,
+            but not defaulted, fields result in MissingAttributeError.
         :param default: the default value to use when the field is missing.
-            Only has effect when the field is not required.
         :param adapter: a function to call to transform and/or validate
             the received value. UnicodeError, ValueError or TypeError from
             this call are reraised as MalformedAttributeError.
@@ -60,8 +59,9 @@ class Field(object):
         if not callable(adapter):
             raise TypeError("Adapter must be callable")
 
-        if isinstance(path, six.string_types):
+        if not isinstance(path, list):
             path = [path]
+
         elif not path:
             raise ValueError('Path cannot be empty')
 
@@ -70,6 +70,17 @@ class Field(object):
         self._default = default
         self._adapter = adapter
 
+    def _get_item(self, dct, key_or_callable, **context):
+        if not callable(key_or_callable):
+            return dct[key_or_callable]
+
+        for candidate_key in dct:
+            if key_or_callable(
+                    candidate_key, value=dct[candidate_key], **context):
+                return dct[candidate_key]
+
+        raise KeyError(key_or_callable)
+
     def _load(self, body, resource, nested_in=None):
         """Load this field from a JSON object.
 
@@ -77,7 +88,8 @@ class Field(object):
         :param resource: ResourceBase instance for which the field is loaded.
         :param nested_in: parent resource path (for error reporting only),
             must be a list of strings or None.
-        :raises: MissingAttributeError if a required field is missing.
+        :raises: MissingAttributeError if a required field is missing
+            and not defaulted.
         :raises: MalformedAttributeError on invalid field value or type.
         :returns: loaded and verified value
         """
@@ -85,18 +97,32 @@ class Field(object):
         for path_item in self._path[:-1]:
             body = body.get(path_item, {})
 
-        if name not in body:
+        try:
+            item = self._get_item(body, name)
+
+        except KeyError:
             if self._required:
                 path = (nested_in or []) + self._path
-                raise exceptions.MissingAttributeError(
-                    attribute='/'.join(path),
-                    resource=resource.path)
-            else:
-                # Do not run the adapter on the default value
-                return self._default
+
+                if self._default is None:
+                    raise exceptions.MissingAttributeError(
+                        attribute='/'.join(path),
+                        resource=resource.path)
+
+                logging.warning(
+                    'Applying default "%s" on required, but missing '
+                    'attribute "%s"' % (self._default, path))
+
+            # Do not run the adapter on the default value
+            return self._default
+
+        # NOTE(etingof): this is just to account for schema violation
+        if item is None:
+            return
 
         try:
-            value = self._adapter(body[name])
+            return self._adapter(item)
+
         except (UnicodeError, ValueError, TypeError) as exc:
             path = (nested_in or []) + self._path
             raise exceptions.MalformedAttributeError(
@@ -104,8 +130,6 @@ class Field(object):
                 resource=resource.path,
                 error=exc)
 
-        return value
-
 
 def _collect_fields(resource):
     """Collect fields from the JSON.
@@ -244,11 +268,10 @@ class MappedField(Field):
             a string or a list of string. In the latter case, the value will
             be fetched from a nested object.
         :param mapping: a mapping to take values from.
-        :param required: whether this field is required. Missing required
-            fields result in MissingAttributeError.
+        :param required: whether this field is required. Missing required,
+            but not defaulted, fields result in MissingAttributeError.
         :param default: the default value to use when the field is missing.
-            Only has effect when the field is not required. This value is not
-            matched against the mapping.
+            This value is not matched against the mapping.
         """
         if not isinstance(mapping, collectionsAbc.Mapping):
             raise TypeError("The mapping argument must be a mapping")
@@ -258,6 +281,51 @@ class MappedField(Field):
             adapter=mapping.get)
 
 
+class MappedListField(Field):
+    """Field taking a list of values with a mapping for the values
+
+    Given JSON {'field':['xxx', 'yyy']}, a sushy resource definition and
+    mapping {'xxx':'a', 'yyy':'b'}, the sushy object to come out will be like
+    resource.field = ['a', 'b']
+    """
+
+    def __init__(self, field, mapping, required=False, default=None):
+        """Create a mapped list field definition.
+
+        :param field: JSON field to fetch the list of values from.
+        :param mapping: a mapping for the list elements.
+        :param required: whether this field is required. Missing required,
+            but not defaulted, fields result in MissingAttributeError.
+        :param default: the default value to use when the field is missing.
+        """
+        if not isinstance(mapping, collectionsAbc.Mapping):
+            raise TypeError("The mapping argument must be a mapping")
+
+        self._mapping_adapter = mapping.get
+        super(MappedListField, self).__init__(
+            field, required=required, default=default,
+            adapter=lambda x: x)
+
+    def _load(self, body, resource, nested_in=None):
+        """Load the mapped list.
+
+        :param body: parent JSON body.
+        :param resource: parent resource.
+        :param nested_in: parent resource name (for error reporting only).
+        :returns: a new list object containing the mapped values.
+        """
+        nested_in = (nested_in or []) + self._path
+        values = super(MappedListField, self)._load(body, resource)
+
+        if values is None:
+            return
+
+        instances = [self._mapping_adapter(value) for value in values
+                     if self._mapping_adapter(value) is not None]
+
+        return instances
+
+
 @six.add_metaclass(abc.ABCMeta)
 class AbstractJsonReader(object):
 
@@ -349,6 +417,7 @@ class ResourceBase(object):
                  connector,
                  path='',
                  redfish_version=None,
+                 registries=None,
                  reader=None):
         """A class representing the base of any Redfish resource
 
@@ -358,12 +427,15 @@ class ResourceBase(object):
         :param path: sub-URI path to the resource.
         :param redfish_version: The version of Redfish. Used to construct
             the object according to schema of the given version.
+        :param registries: Dict of Redfish Message Registry objects to be
+            used in any resource that needs registries to parse messages
         :param reader: Reader to use to fetch JSON data.
         """
         self._conn = connector
         self._path = path
         self._json = None
         self.redfish_version = redfish_version
+        self._registries = registries
         # Note(deray): Indicates if the resource holds stale data or not.
         # Starting off with True and eventually gets set to False when
         # attribute values are fetched.
@@ -376,11 +448,16 @@ class ResourceBase(object):
 
         self.refresh()
 
-    def _parse_attributes(self):
-        """Parse the attributes of a resource."""
+    def _parse_attributes(self, json_doc):
+        """Parse the attributes of a resource.
+
+        Parsed JSON fields are set to `self` as declared in the class.
+
+        :param json_doc: parsed JSON document in form of Python types
+        """
         for attr, field in _collect_fields(self):
             # Hide the Field object behind the real value
-            setattr(self, attr, field._load(self.json, self))
+            setattr(self, attr, field._load(json_doc, self))
 
     def refresh(self, force=True):
         """Refresh the resource
@@ -409,7 +486,7 @@ class ResourceBase(object):
         LOG.debug('Received representation of %(type)s %(path)s: %(json)s',
                   {'type': self.__class__.__name__,
                    'path': self._path, 'json': self._json})
-        self._parse_attributes()
+        self._parse_attributes(self._json)
         self._do_refresh(force)
 
         # Mark it fresh
@@ -461,6 +538,13 @@ class ResourceBase(object):
     def path(self):
         return self._path
 
+    def clone_resource(self, new_resource, path=''):
+        """Instantiate given resource using existing BMC connection context"""
+        return new_resource(
+            self._conn, path or self.path,
+            redfish_version=self.redfish_version,
+            reader=self._reader)
+
     @property
     def resource_name(self):
         return utils.camelcase_to_underscore_joined(self.__class__.__name__)
@@ -477,6 +561,10 @@ class ResourceBase(object):
         return oem.get_resource_extension_by_vendor(
             self.resource_name, vendor, self)
 
+    @property
+    def registries(self):
+        return self._registries
+
 
 @six.add_metaclass(abc.ABCMeta)
 class ResourceCollectionBase(ResourceBase):
@@ -488,7 +576,7 @@ class ResourceCollectionBase(ResourceBase):
                                adapter=utils.get_members_identities)
     """A tuple with the members identities"""
 
-    def __init__(self, connector, path, redfish_version=None):
+    def __init__(self, connector, path, redfish_version=None, registries=None):
         """A class representing the base of any Redfish resource collection
 
         It gets inherited from ``ResourceBase`` and invokes the base class
@@ -497,9 +585,11 @@ class ResourceCollectionBase(ResourceBase):
         :param path: sub-URI path to the resource collection.
         :param redfish_version: The version of Redfish. Used to construct
             the object according to schema of the given version.
+        :param registries: Dict of Redfish Message Registry objects to be
+            used in any resource that needs registries to parse messages.
         """
-        super(ResourceCollectionBase, self).__init__(connector, path,
-                                                     redfish_version)
+        super(ResourceCollectionBase, self).__init__(
+            connector, path, redfish_version, registries)
         LOG.debug('Received %(count)d member(s) for %(type)s %(path)s',
                   {'count': len(self.members_identities),
                    'type': self.__class__.__name__, 'path': self._path})
@@ -520,8 +610,8 @@ class ResourceCollectionBase(ResourceBase):
         :returns: The ``_resource_type`` object
         :raises: ResourceNotFoundError
         """
-        return self._resource_type(self._conn, identity,
-                                   redfish_version=self.redfish_version)
+        return self._resource_type(
+            self._conn, identity, self.redfish_version, self.registries)
 
     @utils.cache_it
     def get_members(self):
